unit TagComment;

interface
uses
  Classes, SysUtils,Tokens,Contnrs,ParseTreeNodeType;

Type

  TTAGContent = TStringList;

  TTAGPattern = class
  private
    FSourceCode: string;
    FTAGType: TCommentStyle;
    FTNType: TParseTreeNodeType;
  public
    property SourceCode : string read FSourcecode;
    property TAGType : TCommentStyle read FTAGType;
    constructor Create(const sc: string; const tt: TCommentStyle; const nt:TParseTreeNodeType );
  end;

  TTAGPatternList = class(TObjectList)
  protected
    function GetItem(const Index: integer): TTAGPattern;
    procedure SetItem(const Index: integer; const item: TTAGPattern);
  public
    property Items[const Index: integer]: TTAGPattern read GetItem write SetItem;
      default;
    function RegisterTAG(const sc : string; const TAGtype :TCommentStyle;
     const NodeType:TParseTreeNodeType) : TTAGPattern;
    function FindItemByName(const st: string): TTAGPattern;
    function FindItemByStyle(const stype: TCommentStyle): TTAGPattern;
    function FindItemByNodeType(const ntype: TParseTreeNodeType): TTAGPattern;
  end;


  TTAGHolderListItem = class
  private
    FTAGItem : TTAGPattern;
    FContent : TTAGContent;
  public
    property TAGItem: TTAGPattern read FTAGItem;
    property Content: TTAGContent read FContent write FContent;
    constructor Create(const tp: TTAGPattern);
    destructor Destroy;override;
  end;

  TTAGHolderList = class(TObjectList)
  private
    function NewTAG(const TAGType:TTAGPattern): TTAGHolderListItem;overload;
    function NewTAG(const TAGstyle:TCommentStyle): TTAGHolderListItem;overload;
    function NewTAG(const NodeType:TParseTreeNodeType): TTAGHolderListItem;overload;
  protected
    function GetItem(const Index: integer): TTAGHolderListItem;
    procedure SetItem(const Index: integer; const item: TTAGHolderListItem);
  public
    property Items[const Index: integer]: TTAGHolderListItem read GetItem write SetItem;
      default;
    procedure AddTAG(const msgTAG: string; const NodeType: TParseTreeNodeType);
    function GetSummary: string;
    function GetContentByStyle(const CommentStyle : TCommentStyle): TTAGContent;
    function GetIndexedLineByStyle(const CommentStyle : TCommentStyle; Index: Integer): string;
    function GetEmptyTAGByStyle(const CommentStyle : TCommentStyle): TTAGContent;
    function FindByType(const CommentStyle : TCommentStyle): TTAGHolderListItem;
    function FindByNodeType(const NodeType : TParseTreeNodeType): TTAGHolderListItem;
    //function GetSubList(const SubList:TObjectList; const CommentStyle : TCommentStyle): TTAGHolderList;
  end;

var
  TAGPatternList : TTAGPatternList;

implementation

resourcestring
  L_EmptyTAG = '#No TAG found in source code#';

var
  EmptyTAG : TTAGContent;


{ TTAGItem }

constructor TTAGPattern.Create(const sc: string; const tt: TCommentStyle; const nt:TParseTreeNodeType );
begin
  FSourcecode := UpperCase(sc);
  FTAGType := tt;
  self.FTNType:=nt;
end;

{ TTAGItemList }

function TTAGPatternList.GetItem(const Index: integer): TTAGPattern;
begin
  Result:=inherited GetItem(Index) as TTAGPattern;
end;

function TTAGPatternList.RegisterTAG(const sc : string; const TAGtype :TCommentStyle;
     const NodeType:TParseTreeNodeType): TTAGPattern;
begin
  Result:=TTAGPattern.Create(sc,TAGType,NodeType);
  Add(result);
end;

function TTAGPatternList.FindItemByName(const st: string): TTAGPattern;
var
  Tmp: TTAGPattern;
  Index: integer;
begin
  Result := nil;
  if Count<=0 then exit;
  Index := 0;
  while ((Index < Count) and (Result = nil)) do
  begin
    Tmp := Items[Index];
    if Assigned(Tmp) then
      if Tmp.SourceCode = st then
        Result := Tmp;
    inc(Index);
  end;
end;

function TTAGPatternList.FindItemByNodeType(
  const ntype: TParseTreeNodeType): TTAGPattern;
var
  Tmp: TTAGPattern;
  Index: integer;
begin
  Result := nil;
  if Count<=0 then exit;
  Index := 0;
  while ((Index < Count) and (Result = nil)) do
  begin
    Tmp := Items[Index];
    if Assigned(Tmp) then
      if Tmp.FTNType = ntype then
        Result := Tmp;
    inc(Index);
  end;
end;


procedure TTAGPatternList.SetItem(const Index: integer; const item: TTAGPattern);
begin
  inherited SetItem(index, item as TObject);
end;

function TTAGPatternList.FindItemByStyle(const stype: TCommentStyle): TTAGPattern;
var
  Tmp: TTAGPattern;
  Index: integer;
begin
  Result := nil;
  if Count<=0 then exit;
  Index := 0;
  while ((Index < Count) and (Result = nil)) do
  begin
    Tmp := Items[Index];
    if Assigned(Tmp) then
      if Tmp.FTAGType = stype then
        Result := Tmp;
    inc(Index);
  end;
end;

{ TTAGHolderListItem }

constructor TTAGHolderListItem.Create(const tp: TTAGPattern);
begin
  FTAGItem:=tp;
  FContent:=TTAGContent.Create;
end;

destructor TTAGHolderListItem.Destroy;
begin
  FreeAndNil(FContent);
  FTAGItem:=nil;
  inherited;
end;

{ TTAGHolderList }

function TTAGHolderList.NewTAG(const TAGType:TTAGPattern): TTAGHolderListItem;
begin
  Result:=TTAGHolderListItem.Create(TAGType);
  Add(Result);
end;

function TTAGHolderList.FindByType(
  const CommentStyle: TCommentStyle): TTAGHolderListItem;
var
  Tmp: TTAGHolderListItem;
  Index: integer;
begin
  Result := nil;
  if Count<=0 then exit;
  Index := 0;
  while ((Index < Count) and (Result = nil)) do
  begin
    Tmp := Items[Index];
    Assert(assigned(Tmp.TAGItem));
    if Assigned(Tmp) then
      if Tmp.TAGItem.TAGType = CommentStyle then
        Result := Tmp;
    inc(Index);
  end;
end;

function TTAGHolderList.GetItem(const Index: integer): TTAGHolderListItem;
begin
  Result:=inherited GetItem(Index) as TTAGHolderListItem;
end;

procedure TTAGHolderList.SetItem(const Index: integer;
  const item: TTAGHolderListItem);
begin
  inherited SetItem(index, item as TObject);
end;

function TTAGHolderList.NewTAG(
  const TAGstyle: TCommentStyle): TTAGHolderListItem;
begin
  Result:=TTAGHolderListItem.Create(TAGPatternList.FindItemByStyle(TAGstyle));
  Add(Result);
end;

procedure TTAGHolderList.AddTAG(const msgTAG: string;
  const NodeType: TParseTreeNodeType);
var
  tmpItem : TTAGHolderListItem;
begin
  tmpItem:=FindByNodeType(NodeType);
  if not assigned(tmpItem) then tmpItem:=NewTAG(NodeType);
  Assert(assigned(tmpItem));
  tmpItem.Content.Add(msgTAG);
end;

function TTAGHolderList.FindByNodeType(
  const NodeType: TParseTreeNodeType): TTAGHolderListItem;
var
  Tmp: TTAGHolderListItem;
  Index: integer;
begin
  Result := nil;
  if Count<=0 then exit;
  Index := 0;
  while ((Index < Count) and (Result = nil)) do
  begin
    Tmp := Items[Index];
    Assert(assigned(Tmp.TAGItem));
    if Assigned(Tmp) then
      if Tmp.TAGItem.FTNType = NodeType then
        Result := Tmp;
    inc(Index);
  end;
end;

function TTAGHolderList.NewTAG(
  const NodeType: TParseTreeNodeType): TTAGHolderListItem;
begin
  Result:=TTAGHolderListItem.Create(TAGPatternList.FindItemByNodeType(NodeType));
  Add(Result);
end;


function TTAGHolderList.GetContentByStyle(
  const CommentStyle: TCommentStyle): TTAGContent;
var
  tmpHolderLIstItem: TTAGHolderListItem;
begin
  tmpHolderLIstItem:=FindByType(CommentStyle);
  if assigned(tmpHolderLIstItem) then
   Result:=tmpHolderLIstItem.Content
  else Result:=nil;
end;

function TTAGHolderList.GetEmptyTAGByStyle(
  const CommentStyle: TCommentStyle): TTAGContent;
begin
  Result:=EmptyTAG;
end;

function TTAGHolderList.GetSummary: string;
var
  tmpHolderLIstItem: TTAGHolderListItem;
begin
  Result:=L_EmptyTAG;
  tmpHolderLIstItem:=FindByType(eDescriptionTAG);
  if assigned(tmpHolderLIstItem) then
   if tmpHolderLIstItem.Content.Count>0 then
    Result:=tmpHolderLIstItem.Content.Strings[0];
end;

function TTAGHolderList.GetIndexedLineByStyle(
  const CommentStyle: TCommentStyle; Index: Integer): string;
var
  tmpContent : TTAGContent;
begin
  tmpContent:= GetContentByStyle(CommentStyle);
  if assigned(tmpContent) then
    if Index<tmpContent.Count then
     result:=tmpContent.Strings[Index]
    else result:=GetEmptyTAGByStyle(CommentStyle).Strings[0]
  else result:=GetEmptyTAGByStyle(CommentStyle).Strings[0];
end;

initialization
  TAGPatternList:=TTAGPatternList.Create(true);
  // Register JavaDoc TAG
  // Default value
  TAGPatternList.RegisterTAG('@DESCRIPTION',eDescriptionTAG,nTAGDescription);
  TAGPatternList.RegisterTAG('@UNKNOWN',eUnknownTAG,nTAGUnknown);
  TAGPatternList.RegisterTAG('@AUTHOR',eAuthorTAG,nTAGAuthor);
  TAGPatternList.RegisterTAG('@VERSION',eVersionTAG,nTAGVersion);
  TAGPatternList.RegisterTAG('@SEE',eSeeTAG,nTAGSee);
  TAGPatternList.RegisterTAG('@PARAM',eParamTAG,nTAGParam);
  TAGPatternList.RegisterTAG('@RETURN',eReturnTAG,nTAGReturn);
  TAGPatternList.RegisterTAG('@TODO',eTodoTAG,nTAGTodo);
  EmptyTAG:=TTAGContent.Create;
  EmptyTAG.Add(L_EmptyTAG)

finalization
  FreeAndNil(EmptyTAG);
  FreeAndNil(TAGPatternList);

end.
