unit AllProcesses;

{ all warnings put together }

{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is AllProcesses, released May 2003.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

interface

uses
  { delphi } Classes,
  ParseTreeNode, BaseVisitor, ConvertTypes;

type

  TAllProcesses = class(TObject)
  private
    fcOnMessages: TStatusMessageProc;
    fcRoot: TParseTreeNode;

    procedure ApplyVisitorType(const pcVisitorType: TTreeNodeVisitorType);

  public
    constructor Create;

    procedure Execute(const pcRoot: TParseTreeNode);

    property OnMessage: TStatusMessageProc read fcOnMessages write fcOnMessages;
  end;

implementation

uses
  { delphi }
  Forms,
  { local }
  {JcfSettings, SetClarify,} VisitSetXY,
  { once-offs }
  //MozComment,
  { obfuscate}
  //FixCase, RemoveComment, RemoveBlankLine, RemoveReturn, ReduceWhiteSpace,
  //RemoveConsecutiveWhiteSpace, RemoveUnneededWhiteSpace, RebreakLines,
  { transform }
  //FindReplace, UsesClauseInsert, UsesClauseRemove, UsesClauseFindReplace,
  { warnings }
  //Warning, WarnEmptyBlock, WarnRealType, WarnAssignToFunctionName,
  //WarnCaseNoElse, WarnDestroy,
  { caps}
  //UnitNameCaps, SpecificWordCaps, Capitalisation,
  { returns }
  //ReturnChars,
  //RemoveReturnsAfterBegin, RemoveReturnsBeforeEnd,
  //PropertyOnOneLine,
  //RemoveBlankLinesAfterProcHeader, RemoveBlankLinesInVars,
  //NoReturnBefore, NoReturnAfter, ReturnBefore, ReturnAfter,
  //BlockStyles, //ReturnsAfterFinalEnd,
  { spacing}
  //NoSpaceAfter, NoSpaceBefore, SingleSpaceBefore, SingleSpaceAfter,
  //SpaceBeforeColon, RemoveSpaceAtLineEnd, VisitStripEmptySpace,
  {indent}
  //VisitSetNesting, Indenter, LongLineBreaker,
  { stats }
  BasicStats,
  VisitStructure,
  VisitComment;
  { align }
  //AlignConst, AlignVars, AlignAssign, AlignTypedef, AlignComment;

constructor TAllProcesses.Create;
begin
  inherited;
  fcOnMessages := nil;
end;


procedure TAllProcesses.ApplyVisitorType(const pcVisitorType: TTreeNodeVisitorType);
var
  lc: TBaseTreeNodeVisitor;
  lsMessage: string;
begin
  Assert(fcRoot <> nil);

  lc := pcVisitorType.Create;
  try
    if lc.IsIncludedInSettings then
    begin
      //if (lc is TWarning) then
      // (lc as TWarning).OnWarning := OnMessage;

      fcRoot.VisitTree(lc);

      // TridenT - No need to send final summary
      if lc.FinalSummary(lsMessage) then
        if lsMessage <> '' then
          OnMessage('', lsMessage, - 1, - 1);
      Application.ProcessMessages;
    end;
  finally
    lc.Free;
  end;
end;

procedure TAllProcesses.Execute(const pcRoot: TParseTreeNode);
begin
  Assert(pcRoot <> nil);
  fcRoot := pcRoot;

  {if FormatSettings.Clarify.OnceOffs = eRunOnly then
  begin
    // run only the once-offs
    OnceOffs;
  end
  else if FormatSettings.Obfuscate.Enabled then
  begin
    Obfuscate;
  end
  else
  begin
    // normal clarify path
    ClarifySetup;
    Transform;
    Warnings;
    Capitalisation;
    LineBreaking;
    Spacing;
    Indent;
    Align;

    // run the once-offs too?
    if FormatSettings.Clarify.OnceOffs = eDoRun then
       OnceOffs;

    // stats last
   }
  //ApplyVisitorType(TBasicStats);
  ApplyVisitorType(TVInterfaceStructure);
  ApplyVisitorType(TVInterfaceComment);

  //end;
end;


end.
