{-------------------------------------------------------------------------------
  Base of all Generator
  The Generator component build a formatted file with the data from the
  extractor. All Generator will descend from the TDOCGenerator describe in this
  unit.

  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
  @Todo      Add ToDo comments (optional)
-------------------------------------------------------------------------------}

unit DocGen;

interface

uses Windows, SysUtils, Classes, Contnrs, Structure, uOptions;

type

  TDOCGenerator = class
  protected
    PreviousItem: TDOCTemplate;
    NextItem: TDOCTemplate;
    FileStructure: TDOCStructure; // Shared object
    procedure WritePageHeader; virtual; abstract;
    procedure WritePageFooter; virtual; abstract;
    procedure WriteLegend; virtual; abstract;
    procedure WriteProjectSection(const DocProject: TDOCStructure); virtual; abstract;
    procedure WriteUnitSection(const DocUnit: TDOCUnit); virtual; abstract;
    procedure WriteClassSection(const DocClass: TDOCClass); virtual; abstract;
    procedure WriteMemberSection(const DocMember: TDOCMember); virtual; abstract;
    procedure WriteBlankSection; virtual; abstract;
    procedure BeforeBuild(const DocItem: TDOCTemplate); virtual; abstract;
    procedure AfterBuild(const DocItem: TDOCTemplate); virtual; abstract;
    procedure AddLine(const st: string); virtual; abstract;
  private
    FDocOptions: TDOCOptions; // Shared Component !
    //FOutputPath: TFilename;
    procedure BuildProjectView;
    procedure BuildUnitsView;
    procedure BuildClassesView(const ParentUnit: TDOCUnit);
    procedure WriteCopyrightSection;
    procedure BuildMembersView(const ParentClass: TDOCClass);
  public
    property DocOptions: TDOCOptions read FDocOptions write FDocOptions;
    //property OutputPath: TFilename read FOutputPath write FOutputPath;
    function Execute(const ds: TDOCStructure): TDOCError; virtual;
  end;

implementation

{ TDOCGenerator }

procedure TDOCGenerator.BuildProjectView;
begin
  BeforeBuild(FileStructure);
  WritePageHeader;
  PreviousItem:=nil;
  NextItem:= nil;
  WriteProjectSection(FileStructure);
  WritePageFooter;
  WriteCopyrightSection;
  AfterBuild(FileStructure);
end;

procedure TDOCGenerator.BuildUnitsView;
var
  IndexUnit: integer;
  tmpUnit: TDOCUnit;
begin
  for IndexUnit := 0 to FileStructure.UnitsCount - 1 do
  begin
    tmpUnit := FileStructure.GetUnit(IndexUnit) as TDOCUnit;
    if IndexUnit<(FileStructure.UnitsCount-1) then NextItem:=FileStructure.GetUnit(IndexUnit+1)
    else NextItem:= nil;
    if IndexUnit>0 then PreviousItem:=FileStructure.GetUnit(IndexUnit-1)
    else PreviousItem:= nil;
    NextItem:= nil;
    BeforeBuild(tmpUnit);
    WritePageHeader;
    WriteUnitSection(tmpUnit);
    WritePageFooter;
    WriteCopyrightSection;
    AfterBuild(tmpUnit);
  end;
end;

procedure TDOCGenerator.BuildClassesView(const ParentUnit: TDOCUnit);
var
  IndexClass: integer;
  tmpClass: TDOCClass;
begin
  for IndexClass := 0 to ParentUnit.ClassesCount - 1 do
  begin
    tmpClass := ParentUnit.GetClass(IndexClass) as TDOCClass;
    if IndexClass<(ParentUnit.ClassesCount-1) then NextItem:=ParentUnit.GetClass(IndexClass+1)
    else NextItem:= nil;
    if IndexClass>0 then PreviousItem:=ParentUnit.GetClass(IndexClass-1)
    else PreviousItem:= nil;
    BeforeBuild(tmpClass);
    WritePageHeader;
    WriteClassSection(tmpClass);
    WritePageFooter;
    WriteCopyrightSection;
    AfterBuild(tmpClass);
  end;
end;

procedure TDOCGenerator.BuildMembersView(const ParentClass: TDOCClass);
var
  IndexMember: integer;
  tmpMember: TDOCMember;
begin
  for IndexMember := 0 to ParentClass.MembersCount - 1 do
  begin
    tmpMember := ParentClass.GetMember(IndexMember) as TDOCMember;
    // Filter member by visibility option
    if tmpMember.MemberVisibility in DocOptions.VisibilitySet then
    begin
      if IndexMember<(ParentClass.MembersCount-1) then NextItem:=ParentClass.GetMember(IndexMember+1)
      else NextItem:= nil;
      if IndexMember>0 then PreviousItem:=ParentClass.GetMember(IndexMember-1)
      else PreviousItem:= nil;
      BeforeBuild(tmpMember);
      WritePageHeader;
      WriteMemberSection(tmpMember);
      WritePageFooter;
      WriteCopyrightSection;
      AfterBuild(tmpMember);
    end;
  end;
end;


function TDOCGenerator.Execute(const ds: TDOCStructure): TDOCError;
var
  tmpUnit: TDOCUnit;
  IndexUnit, IndexClass: integer;
begin
  assert(assigned(ds));
  assert(assigned(DocOptions));
  FileStructure := ds;
  BuildProjectView;
  BuildUnitsView;
  for IndexUnit := 0 to FileStructure.UnitsCount - 1 do
  begin
    tmpUnit := FileStructure.getUnit(IndexUnit) as TDOCUnit;
    BuildClassesView(tmpUnit);
    for IndexClass := 0 to tmpUnit.ClassesCount - 1 do
      BuildMembersView(tmpUnit.GetClass(IndexClass) as TDOCClass);
  end;
  Result := 0;
end;

procedure TDOCGenerator.WriteCopyrightSection;
begin
  //AddLine('-------------------------');
  //AddLine('Copyright 2003 - TridenT ');
end;

end.
