{-------------------------------------------------------------------------------
  HTML Generator
  It provides all classes and functions to build HTML files from the structure
   builder.

  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
  @TODO      Link must be built by the HTMLGenerator and not the HTML canvas
-------------------------------------------------------------------------------}


unit DocGenHTML;

interface

uses Windows, SysUtils, Classes, Contnrs, Structure, DocGen, Forms, Tokens;

type
  // HTML output generator
  TDOCGenHTML = class(TDOCGenerator)
  private
    FGenHTML: TStringList; // Output writer component
    function GetHTMLMemberVisibility(dm: TDOCTemplate): string;
    function GetHTMLIndex: string;
    procedure AddToolBar;
  protected
    procedure WritePageHeader; override;
    procedure WritePageFooter; override;
    procedure WriteLegend; override;
    procedure WriteProjectSection(const DocProject: TDOCStructure); override;
    procedure WriteUnitSection(const DocUnit: TDOCUnit); override;
    procedure WriteClassSection(const DocClass: TDOCClass); override;
    procedure WriteMemberSection(const DocMember: TDOCMember); override;
    procedure WriteBlankSection; override;
    procedure BeforeBuild(const DocItem: TDOCTemplate); override;
    procedure AfterBuild(const DocItem: TDOCTemplate); override;
    procedure AddLine(const st: string); override;
    procedure WriteTAGSection(const TAGType:TCommentStyle;
     const DocItem: TDOCTemplate; const SectionName: string);
  public
    function Execute(const ds: TDOCStructure): TDOCError; override;
  end;

implementation

uses DocGenHTML_Common, uOptions;

{ TDOCGenHTML }

procedure TDOCGenHTML.AddLine(const st: string);
begin
  FGenHTML.Add(st);
end;

procedure TDOCGenHTML.AddToolBar;
var
  PrevLink, NextLink : string;
begin
  if assigned(PreviousItem) then
   PrevLink:=HTMLBuildLink(Format('%s (%s)',[L_Previous,PreviousItem.Name]), PreviousItem.BuildPathName)
  else PrevLink:=L_Previous;
  if assigned(NextItem) then
   NextLink:=HTMLBuildLink(Format('%s (%s)',[L_Next,NextItem.Name]), NextItem.BuildPathName)
  else NextLink:=L_Next;

  AddLine(HTMLGenToolbar(FileStructure.BuildPathName, L_Content,
    GetHTMLIndex, PrevLink, NextLink));
end;

procedure TDOCGenHTML.AfterBuild(const DocItem: TDOCTemplate);
begin
  // Close HTMLFile
  try
    FGenHTML.SaveToFile(DocOptions.OutputFolder + DocItem.BuildPathName + HTML_FILE_EXT);
  finally
    FreeAndNil(FGenHTML);
  end;
end;

procedure TDOCGenHTML.BeforeBuild(const DocItem: TDOCTemplate);
begin
  // Create TextFile
  FGenHTML := TStringList.Create;
end;

function TDOCGenHTML.Execute(const ds: TDOCStructure): TDOCError;
var
  CSSFile: TStringList;
begin
  Result := inherited Execute(ds);

  CSSFile := TStringList.Create;
  CSSFile.Add(HTMLGenCSS);
  try
    CSSFile.SaveToFile(DocOptions.OutputFolder + CSS_FILE + CSS_FILE_EXT);
  finally
    FreeAndNil(CSSFile);
  end;
end;

function TDOCGenHTML.GetHTMLIndex: string;
begin
  Result := HTMLBuildLink(L_Index, FileStructure.BuildPathName);
end;

function TDOCGenHTML.GetHTMLMemberVisibility(dm: TDOCTemplate): string;
begin
  Assert(dm is TDOCMember);
  Result := '  (' + MemberVisibilitySymbol[TDOCMember(dm).MemberVisibility] + ')';
end;

procedure TDOCGenHTML.WriteBlankSection;
begin
  AddLine('');
end;

procedure TDOCGenHTML.WriteClassSection(const DocClass: TDOCClass);
var
  IndexMember: integer;
begin
  // TOOLBAR SECTION
  AddToolBar;
  // CLASS SECTION
  AddLine(HTMLGenSimpleSection(L_Class, DocClass.Name));
  // HIERARCHY SECTION
  AddLine(HTMLGenSimpleSection(L_ClassHierarchy, DocClass.ClassHierarchy));
  // RELATED UNIT  SECTION
  AddLine(HTMLGenSimpleSection(L_RelatedUnit, DocClass.GetParentName));
  // DESCRIPTION SECTION
  WriteTAGSection(eDescriptionTAG,DocClass,L_Description);
  // START OF MEMBERS MULTI SECTION
  AddLine(HTMLGenMultiSectionInit(L_Members));
  //   FIELDS SECTION
  AddLine(HTMLGenTableSectionInit(L_Fields, L_Field, L_Description));
  for IndexMember := 0 to DocClass.FieldsCount - 1 do
  begin
    if TDOCMember(DocClass.GetField(IndexMember)).MemberVisibility in DocOptions.VisibilitySet then
     AddLine(HTMLGenTableSectionLine(HTMLBuildLink(DocClass.GetField(IndexMember).Name
      , DocClass.GetField(IndexMember).BuildPathName) +
      GetHTMLMemberVisibility(DocClass.GetField(IndexMember)),
      DocClass.GetField(IndexMember).TAGHolderList.GetSummary));
  end;
  AddLine(HTMLGenTableSectionEnd);
  //   METHODS SECTION
  AddLine(HTMLGenTableSectionInit(L_Methods, L_Method, L_Description));
  for IndexMember := 0 to DocClass.MethodsCount - 1 do
  begin
    if TDOCMember(DocClass.GetMethod(IndexMember)).MemberVisibility in DocOptions.VisibilitySet then
     AddLine(HTMLGenTableSectionLine(HTMLBuildLink(DocClass.GetMethod(IndexMember).Name
      , DocClass.GetMethod(IndexMember).BuildPathName) +
      GetHTMLMemberVisibility(DocClass.GetMethod(IndexMember))
      , DocClass.GetMethod(IndexMember).TAGHolderList.GetSummary));
  end;
  AddLine(HTMLGenTableSectionEnd);
  //   PROPERTYS SECTION
  AddLine(HTMLGenTableSectionInit(L_Propertys, L_Property, L_Description));
  for IndexMember := 0 to DocClass.PropertysCount - 1 do
  begin
    if TDOCMember(DocClass.GetProperty(IndexMember)).MemberVisibility in DocOptions.VisibilitySet then
     AddLine(HTMLGenTableSectionLine(HTMLBuildLink(DocClass.GetProperty(IndexMember).Name
      , DocClass.GetProperty(IndexMember).BuildPathName) +
      GetHTMLMemberVisibility(DocClass.GetProperty(IndexMember)), L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  //   EVENTS SECTION
  AddLine(HTMLGenTableSectionInit(L_Events, L_Event, L_Description));
  for IndexMember := 0 to DocClass.EventsCount - 1 do
  begin
    if TDOCMember(DocClass.GetEvent(IndexMember)).MemberVisibility in DocOptions.VisibilitySet then
     AddLine(HTMLGenTableSectionLine(HTMLBuildLink(DocClass.GetEvent(IndexMember).Name
      , DocClass.GetEvent(IndexMember).BuildPathName) +
      GetHTMLMemberVisibility(DocClass.GetEvent(IndexMember)), L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // END OF MEMBERS MULTI SECTION
  AddLine(HTMLGenMultiSectionEnd);
  // LEGEND SECTION
  WriteLegend;
  // TOOLBAR SECTION
  AddToolBar;
end;

procedure TDOCGenHTML.WriteLegend;
begin
  // LEGEND SECTION
  AddLine(HTMLGenTableSectionInit(L_Legend, L_Symbol, L_Visibility));
  AddLine(HTMLGenTableSectionLine(MemberVisibilitySymbol[mvPrivate], L_Private));
  AddLine(HTMLGenTableSectionLine(MemberVisibilitySymbol[mvProtected], L_Protected));
  AddLine(HTMLGenTableSectionLine(MemberVisibilitySymbol[mvPublic], L_Public));
  AddLine(HTMLGenTableSectionLine(MemberVisibilitySymbol[mvPublished], L_Published));
  AddLine(HTMLGenTableSectionLine(MemberVisibilitySymbol[mvAutomated], L_Automated));
  AddLine(HTMLGenTableSectionEnd);
end;

procedure TDOCGenHTML.WriteMemberSection(const DocMember: TDOCMember);
var
  tmpMethod: TDOCMethod;
  IndexParameter: integer;
begin
  // TOOLBAR SECTION
  AddToolBar;
  // MEMBER SECTION
  AddLine(HTMLGenSimpleSection(L_Member, DocMember.Name));
  // VISIBILITY SECTION
  AddLine(HTMLGenSimpleSection(L_Visibility
    , MemberVisibilityName.Strings[integer(DocMember.MemberVisibility)]));
  // RELATED CLASS SECTION
  AddLine(HTMLGenSimpleSection(L_RelatedClass, DocMember.GetParentName));
  // DESCRIPTION SECTION
  WriteTAGSection(eDescriptionTAG,DocMember,L_Description);
  // SOURCECODE SECTION
  AddLine(HTMLGenSimpleSection(L_SourceCode, DocMember.BodySource));
  if (DocMember is TDOCMethod) then
  begin
    // PARAMETERS SECTION

    AddLine(HTMLGenTableSectionInit(L_Parameters, L_Parameter, L_Description));
    tmpMethod := DocMember as TDOCMethod;
    for IndexParameter := 0 to tmpMethod.ParametersCount - 1 do
    begin
      AddLine(HTMLGenTableSectionLine(tmpMethod.GetParameter(IndexParameter).Name +
        ' : ' + tmpMethod.GetParameter(IndexParameter).RelatedType
        , tmpMethod.TAGHolderList.GetIndexedLineByStyle(eParamTAG,IndexParameter)));
    end;
    AddLine(HTMLGenTableSectionEnd);
    if tmpMethod.RelatedType <> '' then
    begin
      // RETURN SECTION
      AddLine(HTMLGenSimpleSection(L_Return, tmpMethod.RelatedType));
    end;
  end;
  // LINKS SECTION
  AddLine(HTMLGenSimpleSection(L_Links, L_UnderConstruction));
  // TOOLBAR SECTION
  AddToolBar;
end;

procedure TDOCGenHTML.WritePageFooter;
begin
  AddLine(HTMLGenFooter);
end;

procedure TDOCGenHTML.WritePageHeader;
begin
  // HEADER SECTION
  AddLine(HTMLGenHead(FileStructure.Name));
end;

procedure TDOCGenHTML.WriteProjectSection(const DocProject: TDOCStructure);
var
  IndexUnit: integer;
begin
  // TOOLBAR SECTION
  AddToolBar;
  // PROJECT SECTION
  AddLine(HTMLGenSimpleSection(L_Project, DocProject.Name));
  // SUMMARY SECTION
  AddLine(HTMLGenSimpleSection(L_Summary, L_UnderConstruction));
  // DESCRIPTION SECTION
  AddLine(HTMLGenSimpleColoredSection(L_Description, L_UnderConstruction));
  // AUTHOR SECTION
  AddLine(HTMLGenSimpleSection(L_Author, L_UnderConstruction));
  // FILES/UNITS SECTION
  AddLine(HTMLGenTableSectionInit(L_Files, L_File, L_Description));
  for IndexUnit := 0 to DocProject.UnitsCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine(HTMLBuildLink(DocProject.GetUnit(IndexUnit).Name
      , DocProject.GetUnit(IndexUnit).BuildPathName)
      , DocProject.GetUnit(IndexUnit).TAGHolderList.GetSummary));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // LINKS SECTION
  AddLine(HTMLGenSimpleSection(L_Links, L_UnderConstruction));
  // TOOLBAR SECTION
  AddToolBar;
end;

procedure TDOCGenHTML.WriteTAGSection(const TAGType: TCommentStyle;
  const DocItem: TDOCTemplate; const SectionName: string);
var
  IndexItem: integer;
  TAGStrings : TStringList;
begin
  TAGStrings:=DocItem.TAGHolderList.GetContentByStyle(TAGType);
  if not assigned(TAGStrings) then TAGStrings:=DocItem.TAGHolderList.GetEmptyTAGByStyle(TAGType);
  begin
    AddLine(HTMLGenLinesSectionInit(SectionName));
    for IndexItem := 0 to TAGStrings.Count - 1 do
     AddLine(HTMLGenLinesSectionLineColored(TAGStrings.Strings[IndexItem]));
    AddLine(HTMLGenLinesSectionEnd);
  end;
end;

procedure TDOCGenHTML.WriteUnitSection(const DocUnit: TDOCUnit);
var
  IndexItem: integer;
begin
  // TOOLBAR SECTION
  AddToolBar;
  // UNIT SECTION
  AddLine(HTMLGenSimpleSection(L_Unit, DocUnit.Name));
  // DESCRIPTION SECTION
  WriteTAGSection(eDescriptionTAG,DocUnit,L_Description);
  // AUTHOR SECTION
  WriteTAGSection(eAuthorTAG,DocUnit,L_Author);
  // REVISION SECTION
  WriteTAGSection(eVersionTAG,DocUnit,L_Version);
  // CLASSES SECTION
  AddLine(HTMLGenTableSectionInit(L_Classes, L_Class, L_Description));
  for IndexItem := 0 to DocUnit.ClassesCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine(HTMLBuildLink(TDOCClass(DocUnit.GetClass(IndexItem)).Name
      , DocUnit.GetClass(IndexItem).BuildPathName)
      , DocUnit.GetClass(IndexItem).TAGHolderList.GetSummary));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // TYPES SECTION
  AddLine(HTMLGenTableSectionInit(L_Types, L_Type, L_Description));
  for IndexItem := 0 to DocUnit.TypesCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine({HTMLBuildLink(}DocUnit.GetType(IndexItem).Name
      {,DocUnit.GetType(IndexItem).BuildPathName)}, L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // CONSTANTS SECTION
  AddLine(HTMLGenTableSectionInit(L_Constants, L_Constant, L_Description));
  for IndexItem := 0 to DocUnit.ConstantsCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine({HTMLBuildLink(}DocUnit.GetConstant(IndexItem).Name
      {,DocUnit.GetConstant(IndexItem).BuildPathName)}, L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // VARIABLES SECTION
  AddLine(HTMLGenTableSectionInit(L_Variables, L_Variable, L_Description));
  for IndexItem := 0 to DocUnit.VarsCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine({HTMLBuildLink(}DocUnit.GetVar(IndexItem).Name
      {,DocUnit.GetVar(IndexItem).BuildPathName)}, L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // FUNCTIONS SECTION
  AddLine(HTMLGenTableSectionInit(L_Fonctions, L_Fonction, L_Description));
  for IndexItem := 0 to DocUnit.FunctionsCount - 1 do
  begin
    AddLine(HTMLGenTableSectionLine({HTMLBuildLink(}DocUnit.GetFunction(IndexItem).Name
      {,DocUnit.GetFunction(IndexItem).BuildPathName)}, L_UnderConstruction));
  end;
  AddLine(HTMLGenTableSectionEnd);
  // LINKS SECTION
  AddLine(HTMLGenSimpleSection(L_Links, L_UnderConstruction));
  // TOOLBAR SECTION
  AddToolBar;
end;

end.
