{-------------------------------------------------------------------------------
  TeXT Generator
  It provides all class and function to build Text files from the structure
   builder. It is mainly used for debug purpose.

  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit DocGenTXT;

interface

uses Windows, SysUtils, Classes, Contnrs, Structure, DocGen, Forms;

type

  TDOCGenTXT = class(TDOCGenerator)
  private
    FGenText: TStringList;
  protected
    procedure WritePageHeader; override;
    procedure WritePageFooter; override;
    procedure WriteLegend; override;
    procedure WriteProjectSection(const DocProject: TDOCStructure); override;
    procedure WriteUnitSection(const DocUnit: TDOCUnit); override;
    procedure WriteClassSection(const DocClass: TDOCClass); override;
    procedure WriteMemberSection(const DocMember: TDOCMember); override;
    procedure WriteBlankSection; override;
    procedure BeforeBuild(const DocItem: TDOCTemplate); override;
    procedure AfterBuild(const DocItem: TDOCTemplate); override;
    procedure AddLine(const st: string); override;
  end;

implementation

uses uOptions;

const
  TMemberVisibilityTxt: array[Low(TMemberVisibility)..High(TMemberVisibility)] of
  char = ('-', '#', '+', '*', 'A');


  { TDOCGenTXT }

procedure TDOCGenTXT.AddLine(const st: string);
begin
  FGenText.Add(st);
end;

procedure TDOCGenTXT.AfterBuild(const DocItem: TDOCTemplate);
begin
  // Close TextFile
  try
    FGenText.SaveToFile(DocOptions.OutputFolder + DocItem.BuildPathName + '.txt');
  finally
    FreeAndNil(FGenText);
  end;
end;

procedure TDOCGenTXT.BeforeBuild(const DocItem: TDOCTemplate);
begin
  // Create TextFile
  FGenText := TStringList.Create;
end;

procedure TDOCGenTXT.WriteBlankSection;
begin
  AddLine('');
end;

procedure TDOCGenTXT.WriteClassSection(const DocClass: TDOCClass);
var
  IndexMember: integer;
  tmpMember: TDOCMember;
begin
  AddLine('Class : ' + DocClass.Name);
  WriteBlankSection;
  AddLine('Class Hierachy');
  AddLine(' ' + DocClass.ClassHierarchy);
  WriteBlankSection;
  AddLine('Members');
  WriteBlankSection;
  AddLine(Format(' Fields (%d)', [DocClass.FieldsCount]));
  for IndexMember := 0 to DocClass.MembersCount - 1 do
  begin
    tmpMember := DocClass.GetMember(IndexMember) as TDOCMember;
    if tmpMember is TDOCField then
      AddLine('  ' + TMemberVisibilityTxt[tmpMember.MemberVisibility] + tmpMember.Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Methods (%d)', [DocClass.MethodsCount]));
  for IndexMember := 0 to DocClass.MembersCount - 1 do
  begin
    tmpMember := DocClass.GetMember(IndexMember) as TDOCMember;
    if tmpMember is TDOCMethod then
      AddLine('  ' + TMemberVisibilityTxt[tmpMember.MemberVisibility] + tmpMember.Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Propertys (%d)', [DocClass.PropertysCount]));
  for IndexMember := 0 to DocClass.MembersCount - 1 do
  begin
    tmpMember := DocClass.GetMember(IndexMember) as TDOCMember;
    if tmpMember is TDOCProperty then
      AddLine('  ' + TMemberVisibilityTxt[tmpMember.MemberVisibility] + tmpMember.Name);
  end;
  WriteBlankSection;
  AddLine('Related unit : ' + DocClass.GetParentName);
  WriteBlankSection;
  WriteLegend;
  WriteBlankSection;
end;

procedure TDOCGenTXT.WriteLegend;
begin
  AddLine('Legend');
  AddLine(' - Private');
  AddLine(' # Protected');
  AddLine(' + Public');
  AddLine(' * Published');
  AddLine(' A Automated');
end;

procedure TDOCGenTXT.WriteMemberSection(const DocMember: TDOCMember);
var 
  tmpMethod: TDOCMethod;
  IndexParameter: integer;
begin
  AddLine('Member : ' + DocMember.Name);
  WriteBlankSection;
  AddLine('Visibility : ' + MemberVisibilityName.Strings[integer(DocMember.MemberVisibility)]);
  WriteBlankSection;
  AddLine('SourceCode : ');
  if (DocMember is TDOCField) or (DocMember is TDOCProperty) then
    AddLine('  ' + DocMember.BodySource)
  else
    AddLine('  ' + DocMember.BodySource);
  WriteBlankSection;

  if (DocMember is TDOCMethod) then
  begin
    tmpMethod := DocMember as TDOCMethod;
    AddLine(Format('Parameters (%d)', [tmpMethod.ParametersCount]));
    for IndexParameter := 0 to tmpMethod.ParametersCount - 1 do
      AddLine('  ' + tmpMethod.GetParameter(IndexParameter).Name + ' : ' +
        tmpMethod.GetParameter(IndexParameter).RelatedType);
    //else AddLine(' no parameters');
    WriteBlankSection;

    if tmpMethod.RelatedType <> '' then
    begin
      AddLine('Returns : ');
      Addline(' ' + tmpMethod.RelatedType);
      WriteBlankSection;
    end;
  end;
  AddLine('Related class : ' + DocMember.GetParentName);
  WriteBlankSection;
end;

procedure TDOCGenTXT.WritePageFooter;
begin
  WriteBlankSection;
  AddLine('------------------------------------------');
  AddLine('DelphiCodeToDoc - CopyRight 2003 - TridenT');
end;

procedure TDOCGenTXT.WritePageHeader;
begin
  AddLine(FileStructure.Name);
  AddLine('------------------------------------------');
  WriteBlankSection;
end;

procedure TDOCGenTXT.WriteProjectSection(const DocProject: TDOCStructure);
var
  nbClass, IndexUnit: integer;
begin
  nbClass := 0;
  for IndexUnit := 0 to DocProject.UnitsCount - 1 do
    inc(nbClass, TDOCUnit(DocProject.GetUnit(IndexUnit)).ClassesCount);

  AddLine('Project : ' + DocProject.Name);
  AddLine(Format(' %d class in %d units.', [nbClass, DocProject.UnitsCount]));
  WriteBlankSection;
end;

procedure TDOCGenTXT.WriteUnitSection(const DocUnit: TDOCUnit);
var
  IndexItem: integer;
begin
  AddLine('Unit : ' + DocUnit.Name);
  WriteBlankSection;
  AddLine(Format(' Classes (%d)', [DocUnit.ClassesCount]));
  for IndexItem := 0 to DocUnit.ClassesCount - 1 do
  begin
    AddLine('  ' + TDOCClass(DocUnit.GetClass(IndexItem)).Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Types (%d)', [DocUnit.TypesCount]));
  for IndexItem := 0 to DocUnit.TypesCount - 1 do
  begin
    AddLine('  ' + (DocUnit.GetType(IndexItem)).Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Constants (%d)', [DocUnit.ConstantsCount]));
  for IndexItem := 0 to DocUnit.ConstantsCount - 1 do
  begin
    AddLine('  ' + (DocUnit.GetConstant(IndexItem)).Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Vars (%d)', [DocUnit.VarsCount]));
  for IndexItem := 0 to DocUnit.VarsCount - 1 do
  begin
    AddLine('  ' + (DocUnit.GetVar(IndexItem)).Name);
  end;
  WriteBlankSection;
  AddLine(Format(' Functions (%d)', [DocUnit.FunctionsCount]));
  for IndexItem := 0 to DocUnit.FunctionsCount - 1 do
  begin
    AddLine('  ' + (DocUnit.GetFunction(IndexItem)).Name);
  end;
  WriteBlankSection;
end;

end.
