{-------------------------------------------------------------------------------
  Create the structure of source code component from the result of the Parser
  All tools used by the Builder are in this unit.
  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit Structure;

interface

uses Windows, SysUtils, Classes, Contnrs, ComCtrls,TAGComment, uOptions;

type
  TDOCError         = integer;
  TMethodBind       = (mtStatic, mtDynamic, mtVirtual, mtAbstract);
  TMethodBindSet    = set of TMethodBind;
  TMethodType       = (mtProcedure, mtFunction, mtConstructor, mtDestructor);
  TMethodDirectives = (mdExternal, mdPascal, mdSafecall, mdAbstract,
    mdAutomated, mdFar, mdStdcall, mdAssembler, mdInline, mdForward,
    mdVirtual, mdCdecl, mdMessage, mdName, mdRegister, mdDispId,
    mdNear, mdDynamic, mdExport, mdOverride, mdResident, mdLocal, mdOverload,
    mdReintroduce, mdDeprecated, mdLibrary, mdPlatform);
  TMethodDirectivesSet = set of TMethodDirectives;

  TDOCTemplate     = class;
  TDOCTemplateList = class;

  TDOCLinkList      = TDOCTemplateList;  // of TDOCLinks
  TDOCClassList     = TDOCTemplateList; // of TDOCClasses
  TDOCTypeList      = TDOCTemplateList;  // of TDOCTypes
  TDOCConstantList  = TDOCTemplateList;  // of TDOCConst
  TDOCFieldList     = TDOCTemplateList; // of TDOCField
  TDOCPropertyList  = TDOCTemplateList; // of TDOCPropertie
  TDOCEventList     = TDOCTemplateList; // of TDOCEvent
  TDOCParameterList = TDOCTemplateList; // of TDOCparameters
  TDOCUnitList      = TDOCTemplateList;  // of TDOCUnit
  TDOCMethodList    = TDOCTemplateList; // of TDOCMethod
  TDOCMemberList    = TDOCTemplateList; // of TDOCMember;
  TDOCVarList       = TDOCTemplateList; // of TDOCVar
  TDOCFunctionList  = TDOCTemplateList; // of TDOCFunction

  //  A link to a unit, class or another Maybe replace by an ObjectList with all DocTemplate !
  TDOCLink = class
  private
    Name: string;
    Link: TDOCTemplate;
  public
    constructor Create(const ln: string);
  end;

  // Base class for Classes, members, ... in the final structure
  TDOCTemplate = class(TObject)
  private
    FRelatedParent: TDocTemplate; // Logical Parent of this item
    FName: string;  // Name of the item, often time an identifier
    FTAGHolderList: TTAGHolderList; // Holder for all item's TAG
  public
    RelatedType: string; // TDOCLink;
    BodySource: string; // formatted Source code of the item
    Links: TDOCLinkList;  // of TDOCLinks
    property Name: string read FName write FName;
    property TAGHolderList: TTAGHolderList read FTAGHolderList write FTAGHolderList;
    constructor Create(const st: string; const rp: TDocTemplate); virtual;
    destructor Destroy; override;
    function AddLink(const ln: string): TDOCLink;
    function GetParentName: string;
    function BuildPathName: string;
  end;

  // Base class for TDOCTemplate containers
  TDOCTemplateList = class(TObjectList)
  protected
    function GetItem(const Index: integer): TDOCTemplate;
    procedure SetItem(const Index: integer; const item: TDOCTemplate);
  public
    property Items[const Index: integer]: TDOCTemplate read GetItem write SetItem;
      default;
    function FindItemByName(const st: string): TDOCTemplate;
    function NextItem(const Item: TDOCTemplate): TDOCTemplate;
    function PrevItem(const Item: TDOCTemplate): TDOCTemplate;
  end;

  // Specialized TDOCTemplate class for Unit structure
  TDOCUnit = class(TDOCTemplate)
  private
    FileName: string; // Related Filename of the unit, generally 'UnitName.pas'
    Classes: TDOCClassList; // of TDOCClasses
    Types: TDOCTypeList;  // of TDOCTypes
    Constants: TDOCConstantList;  // of TDOCConst
    Vars: TDOCVarList; // of TDOCInterface
    Functions: TDOCFunctionList; // of TDOCFunction
  public
    function ClassesCount: integer;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    function AddClass(const cname: string): TDOCTemplate;
    function GetClass(const index: integer): TDOCTemplate;
    function FindClass(const cname: string): TDOCTemplate;
    function AddFunction(const fname: string): TDOCTemplate;
    function GetFunction(index: integer): TDOCTemplate;
    function FunctionsCount: integer;
    function AddType(const tname: string): TDOCTemplate;
    function GetType(index: integer): TDOCTemplate;
    function TypesCount: integer;
    function AddVar(const vname: string): TDOCTemplate;
    function GetVar(index: integer): TDOCTemplate;
    function VarsCount: integer;
    function AddConstant(const cname: string): TDOCTemplate;
    function GetConstant(index: integer): TDOCTemplate;
    function ConstantsCount: integer;
    procedure SortContent;
  end;

  // Specialized TDOCTemplate class for Class structure
  TDOCVar      = class(TDocTemplate);
  // Specialized TDOCTemplate class for Type structure
  TDOCType     = class(TDocTemplate);
  // Specialized TDOCTemplate class for Constant structure
  TDOCConstant = class(TDocTemplate);
  // Specialized TDOCTemplate class for Function structure
  TDOCFunction = class(TDocTemplate);

  // Specialized TDOCTemplate class for Class structure
  TDOCClass = class(TDOCTemplate)
  private
    Members: TDOCMemberList; // of TDOCMember
    Fields: TDOCFieldList; // of TDOCField
    Propertys: TDOCPropertyList; // of TDOCPropertie
    Methods: TDOCMethodList; // of TDOCMethod
    Events: TDOCEventList; // of TDOCEvent
  public
    ClassHierarchy: string; // Mainly parent class
    constructor Create(const st: string; const rp: TDOCTemplate); override;
    destructor Destroy; override;
    function AddField(const fname: string): TDOCTemplate;
    function AddMethod(const mname: string): TDOCTemplate;
    function AddProperty(const pname: string): TDOCTemplate;
    function GetField(const index: integer): TDOCTemplate;
    function GetProperty(const index: integer): TDOCTemplate;
    function GetMethod(const index: integer): TDOCTemplate;
    function GetMember(const index: integer): TDOCTemplate;
    function GetEvent(const index: integer): TDOCTemplate;
    function FieldsCount: integer;
    function PropertysCount: integer;
    function MethodsCount: integer;
    function MembersCount: integer;
    function EventsCount: integer;
    function FindField(const fname: string): TDOCtemplate;
    function FindMember(const fname: string): TDOCtemplate;
    procedure SortContent;
  end;

  // Specialized TDOCTemplate class for Member structure
  TDOCMember = class(TDOCTemplate)
  public
    MemberVisibility: TMemberVisibility; // Visibility of the Member
  end;

  // Specialized TDOCMember class for Method structure with parameter support
  TDOCMethod = class(TDOCMember)
  private
    Parameters: TDOCParameterList; // List of parameters in the method
  public
    MethodType: TMethodType; // Internal type of method (constructor, procedure, etc...)
    MethodDirectiveSet: TMethodDirectivesSet; // Set of directive used in the method
    MethodBind: TMethodBind; // Binding method
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    function AddParameter(const pname: string): TDOCTemplate;
    function GetParameter(const index: integer): TDOCTemplate;
    function ParametersCount: integer;
  end;

  // Specialized TDOCMember class for Field structure
  TDOCField    = class(TDOCMember);
  // Specialized TDOCMember class for Property structure
  TDOCProperty = class(TDOCMember);
  // Specialized TDOCMember class for Event structure
  TDOCEvent    = class(TDOCMember);
  // Specialized TDOCTemplate class for Parameter structure
  TDOCParameter = class(TDOCTemplate);

  // Specialized TDOCTemplate class for the whole structure itself
  TDOCStructure = class(TDOCTemplate)
  private
    Units: TDOCUnitList;  // List of all units in the structure
    FDOCOptions: TDOCOptions; // All Options. Beware! Shared Component.
  public
    property DocOptions: TDOCOptions read FDOCOptions write FDOCOptions;
    constructor Create(const st: string; const rp: TDocTemplate); override;
    destructor Destroy; override;
    function UnitsCount: integer;
    function AddUnit(const uname: string): TDOCTemplate;
    function GetUnit(const index: integer): TDOCTemplate;
    function FindUnit(const uname: string): TDOCTemplate;
    procedure ConvertToTreeNode(const tnList: TTreeNodes);
    procedure SortContent;
    procedure Clear;
  end;

var
  MemberVisibilityName: TStringList;

const
  MethodeTypeName: array[low(TMethodType)..High(TMethodType)] of string =
    ('Procedure', 'Function', 'Constructor', 'Destructor');
  //TMemberVisibility = (mvPrivate, mvProtected, mvPublic, mvPublished, mvAutomated);
  MemberVisibilitySymbol: array[low(TMemberVisibility)..High(TMemberVisibility)]
  of string = ('-', '#', '+', '*', 'A');

implementation

// Comparison function for TDocTemplateList
function DocTemplateListSortCompare(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TDOCtemplate(Item1).Name, TDOCTemplate(Item2).Name)
end;

function TDOCTemplate.AddLink(const ln: string): TDOCLink;
begin
  Result := TDOCLink.Create(ln);
  Links.Add(Result);
end;

{-------------------------------------------------------------------------------
  Build a base filename with all parent's name separated with underscore
  It is mainly used to produce the output filename of this item's documentation.
  @return parent's name concatenation separated with underscore
-------------------------------------------------------------------------------}
function TDOCTemplate.BuildPathName: string;
var
  tmpDT: TDocTemplate;
begin
  Result := FName;
  tmpDT  := self;
  while tmpDT.FRelatedParent <> nil do
  begin
    tmpDT := tmpDT.FRelatedParent;
    insert(tmpDT.Name + '_', Result, 1);
  end;
end;

{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCTemplate object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list
  @param  st  Name of the new item (mainly its identifier)
  @param  rp  Related logical parent and owner
-------------------------------------------------------------------------------}
constructor TDOCTemplate.Create(const st: string; const rp: TDocTemplate);
begin
  inherited Create;
  FTAGHolderList:= TTAGHolderList.Create(true);
  FName          := st;
  BodySource     := '';
  Links          := TDOCLinkList.Create(False); // links are independants
  FRelatedParent := rp;
end;

//  Destroys of an instance of a TDOCTemplate object
destructor TDOCTemplate.Destroy;
begin
  FreeAndNil(FTAGHolderList);
  FreeAndNil(Links);
  FRelatedParent := nil;
  inherited;
end;

{-------------------------------------------------------------------------------
  Return the name of the parent (owner)
  If no parent is assigned, then return the specified value '#NO-PARENT#'
  @return Parent and owner's item
-------------------------------------------------------------------------------}
function TDOCTemplate.GetParentName: string;
begin
  if FRelatedParent <> nil then
    Result := FRelatedParent.Name
  else 
    Result := '#NO-PARENT#';
end;

{-------------------------------------------------------------------------------
  Add a Class to the structure
  @param  cname New class name (its identifier)
  @return Newly created reference for the class
-------------------------------------------------------------------------------}
function TDOCUnit.AddClass(const cname: string): TDOCTemplate;
begin
  Result := TDOCClass.Create(cname, self);
  Classes.Add(Result);
end;

{-------------------------------------------------------------------------------
  Add a Function to the structure
  @param  fname New function name (its identifier)
  @return Newly created reference for the function
-------------------------------------------------------------------------------}
function TDOCUnit.AddFunction(const fname: string): TDOCTemplate;
begin
  Result := TDOCFunction.Create(fname, self);
  Functions.Add(Result);
end;

{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCUnit object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Unit (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the Structure it self
-------------------------------------------------------------------------------}
constructor TDOCUnit.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  Classes   := TDOCClassList.Create(True);
  Constants := TDOCConstantList.Create(True);
  Types     := TDOCTypeList.Create(True);
  Vars      := TDOCVarList.Create(True);
  Functions := TDOCFunctionList.Create(True);
end;

//  Destroys of an instance of a TDOCUnit object
destructor TDOCUnit.Destroy;
begin
  FreeAndNil(Classes);
  FreeAndNil(Constants);
  FreeAndNil(Types);
  FreeAndNil(Vars);
  FreeAndNil(Functions);
  inherited;
end;

{-------------------------------------------------------------------------------
  Find a class reference in the structure by its name
  It simply call the FindItemByName of the list.
  @param  cname Name of the class to find
  @return Class reference if succesfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.FindClass(const cname: string): TDOCTemplate;
begin
  Result := Classes.FindItemByName(cname);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Class specified by its index in the Classes List
  @param  index ParameterDescription
  @return Class reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.GetClass(const index: integer): TDOCTemplate;
begin
  Result := Classes.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Return number of Classes in the list
  @return Number of classes
-------------------------------------------------------------------------------}
function TDOCUnit.ClassesCount: integer;
begin
  Result := Classes.Count;
end;

{-------------------------------------------------------------------------------
  Add a constant to the structure
  @param  cname New constant name (its identifier)
  @return Newly created reference for the constant
-------------------------------------------------------------------------------}
function TDOCUnit.AddConstant(const cname: string): TDOCTemplate;
begin
  Result := TDOCConstant.Create(cname, self);
  Constants.Add(Result);
end;

{-------------------------------------------------------------------------------
  Add a Type to the structure
  @param  tname New Type name (its identifier)
  @return Newly created reference for the Type
-------------------------------------------------------------------------------}
function TDOCUnit.AddType(const tname: string): TDOCTemplate;
begin
  Result := TDOCType.Create(tname, self);
  Types.Add(Result);
end;

{-------------------------------------------------------------------------------
  Add a Variable to the structure
  @param  vname New Variable name (its identifier)
  @return Newly created reference for the variable
-------------------------------------------------------------------------------}
function TDOCUnit.AddVar(const vname: string): TDOCTemplate;
begin
  Result := TDOCVar.Create(vname, self);
  Vars.Add(Result);
end;

{-------------------------------------------------------------------------------
  Return number of Constants in the list
  @return Number of constants
-------------------------------------------------------------------------------}
function TDOCUnit.ConstantsCount: integer;
begin
  Result := Constants.Count;
end;

{-------------------------------------------------------------------------------
  Return number of Functions in the list
  @return Number of functions
-------------------------------------------------------------------------------}
function TDOCUnit.FunctionsCount: integer;
begin
  Result := Functions.Count;
end;

{-------------------------------------------------------------------------------
  Get the reference of the Constant specified by its index in the Constants List
  @param  index Index in the Constants List
  @return Constant reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.GetConstant(index: integer): TDOCTemplate;
begin
  Result := Constants.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Function specified by its index in the functions List
  @param  index Index in the Functions List
  @return Function reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.GetFunction(index: integer): TDOCTemplate;
begin
  Result := Functions.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Type specified by its index in the Types List
  @param  index Index in the Types List
  @return Type reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.GetType(index: integer): TDOCTemplate;
begin
  Result := Types.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Variable specified by its index in the Vars List
  @param  index Index in the Vars List
  @return Variable reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCUnit.GetVar(index: integer): TDOCTemplate;
begin
  Result := Vars.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Return number of Types in the list
  @return Number of types
-------------------------------------------------------------------------------}
function TDOCUnit.TypesCount: integer;
begin
  Result := Types.Count;
end;

{-------------------------------------------------------------------------------
  Return number of Variables in the list
  @return Number of varibales
-------------------------------------------------------------------------------}
function TDOCUnit.VarsCount: integer;
begin
  Result := Vars.Count;
end;

// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCUnit.SortContent;
var
  Index : integer;
begin
  Classes.Sort(DocTemplateListSortCompare);
  for Index:=0 to Classes.Count-1 do
   TDOCClass(Classes.Items[Index]).SortContent;

  Constants.Sort(DocTemplateListSortCompare);
  Functions.Sort(DocTemplateListSortCompare);
  Types.Sort(DocTemplateListSortCompare);
  Vars.Sort(DocTemplateListSortCompare);
end;

{-------------------------------------------------------------------------------
  Add a Field to the structure
  @param  fname New Field name (its identifier)
  @return Newly created reference for the Field
-------------------------------------------------------------------------------}
function TDOCClass.AddField(const fname: string): TDOCTemplate;
begin
  Result := TDOCField.Create(fname, self);
  Fields.Add(Result);
  Members.Add(Result);
end;

{-------------------------------------------------------------------------------
  Add a Method to the structure
  @param  mname New Method name (its identifier)
  @return Newly created reference for the class
-------------------------------------------------------------------------------}
function TDOCClass.AddMethod(const mname: string): TDOCTemplate;
begin
  Result := TDOCMethod.Create(mname, self);
  Methods.Add(Result);
  Members.Add(Result);
end;

{-------------------------------------------------------------------------------
  Add a Property to the structure
  @param  pname New Property name (its identifier)
  @return Newly created reference for the class
-------------------------------------------------------------------------------}
function TDOCClass.AddProperty(const pname: string): TDOCTemplate;
begin
  Result := TDOCProperty.Create(pname, self);
  Propertys.Add(Result);
  Members.Add(Result);
end;

{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCCLass object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Class (mainly its identifier)
  @param  rp  Related logical parent and owner, the unit it written into
-------------------------------------------------------------------------------}
constructor TDOCClass.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  ClassHierarchy := ''; //nil;
  Events         := TDOCEventList.Create(True);
  Fields         := TDOCFieldList.Create(True);
  Methods        := TDOCMethodList.Create(True);
  Propertys      := TDOCPropertyList.Create(True);
  // Member list is the union of previous list, so not owner of objects
  Members := TDOCMemberList.Create(False);
end;

//  Destroys of an instance of a TDOCCLass object
destructor TDOCClass.Destroy;
begin
  FreeAndNil(Members);
  FreeAndNil(Events);
  FreeAndNil(Fields);
  FreeAndNil(Methods);
  FreeAndNil(Propertys);
  inherited;
end;

{-------------------------------------------------------------------------------
  Find a Field reference in the structure by its name
  It simply call the FindItemByName of the list.
  @param  fname Name of the field to find
  @return Field reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.FindField(const fname: string): TDOCtemplate;
begin
  Result := Fields.FindItemByName(fname);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Field specified by its index in the Fileds List
  @param  index Index in the Fields List
  @return Field reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.GetField(const index: integer): TDOCTemplate;
begin
  Result := Fields.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Return number of Fields in the list
  @return Number of fields
-------------------------------------------------------------------------------}
function TDOCClass.FieldsCount: integer;
begin
  Result := Fields.Count;
end;

{-------------------------------------------------------------------------------
  Get the reference of the Method specified by its index in the Methods List
  @param  index Index in the Methods List
  @return Method reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.GetMethod(const index: integer): TDOCTemplate;
begin
  Result := Methods.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Property specified by its index in the Propertys List
  @param  index Index in the Propertys List
  @return Property reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.GetProperty(const index: integer): TDOCTemplate;
begin
  Result := Propertys.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Member specified by its index in the Members List
  @param  index Index in the Members List
  @return Member reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.GetMember(const index: integer): TDOCTemplate;
begin
  Result := Members.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Return number of Methods in the list
  @return Number of methods
-------------------------------------------------------------------------------}
function TDOCClass.MethodsCount: integer;
begin
  Result := Methods.Count;
end;

{-------------------------------------------------------------------------------
  Return number of Propertys in the list
  @return Number of propertys
-------------------------------------------------------------------------------}
function TDOCClass.PropertysCount: integer;
begin
  Result := Propertys.Count;
end;

{-------------------------------------------------------------------------------
  Return number of Members in the list
  @return Number of members
-------------------------------------------------------------------------------}
function TDOCClass.MembersCount: integer;
begin
  Result := Members.Count;
end;

{-------------------------------------------------------------------------------
  Return number of Events in the list
  @return Number of events
-------------------------------------------------------------------------------}
function TDOCClass.EventsCount: integer;
begin
  Result := Events.Count;
end;

{-------------------------------------------------------------------------------
  Get the reference of the Event specified by its index in the Events List
  @param  index Index in the Events List
  @return Event reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.GetEvent(const index: integer): TDOCTemplate;
begin
  Result := Events.GetItem(index);
end;

// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCClass.SortContent;
begin
  Members.Sort(DocTemplateListSortCompare);
  Fields.Sort(DocTemplateListSortCompare);
  Methods.Sort(DocTemplateListSortCompare);
  Events.Sort(DocTemplateListSortCompare);
  Propertys.Sort(DocTemplateListSortCompare);
end;

{-------------------------------------------------------------------------------
  Find a member reference in the structure by its name
  It simply call the FindItemByName of the list.
  Member can be Field, Method, Property or Event
  @param  fname Name of the member to find
  @return Member reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCClass.FindMember(const fname: string): TDOCtemplate;
begin
  Result:=Members.FindItemByName(fName);
end;

{-------------------------------------------------------------------------------
  Add a Unit to the structure
  @param  uname New Unit name (its identifier)
  @return Newly created reference for the class
-------------------------------------------------------------------------------}
function TDOCStructure.AddUnit(const uname: string): TDOCTemplate;
begin
  Result := TDOCUnit.Create(uname, self);
  Units.Add(Result);
end;

// Clear all units
procedure TDOCStructure.Clear;
begin
  Units.Clear;
end;

{-------------------------------------------------------------------------------
  Converts the structure into a treenode
  The structure has units, which hold classes, which hold fields and methods
   and events and propertys ... So, it can be represented has a treenode.
  The TreeNode wil be cleared before filling.
  @param      tnList An initiazied TreeNode
-------------------------------------------------------------------------------}
procedure TDOCStructure.ConvertToTreeNode(const tnList: TTreeNodes);
var
  tnRoot, tnUnit, tnClass: TTreeNode;
  IndexUnit, IndexClass: integer;
  tmpClass, tmpUnit: TDOCTemplate;
begin
  Assert(tnList <> nil);
  // Clear the List
  tnList.Clear;
  // Add Root project
  tnRoot := tnList.Add(nil, FName);
  // Add Units
  for IndexUnit := 0 to Units.Count - 1 do
  begin
    tmpUnit     := GetUnit(IndexUnit);
    tnUnit      := tnList.AddChild(tnRoot, tmpUnit.FName);
    tnUnit.Data := tmpUnit;
    for IndexClass := 0 to (tmpUnit as TDOCUnit).ClassesCount - 1 do
    begin
      tmpClass     := (tmpUnit as TDOCUnit).GetClass(IndexClass);
      tnClass      := tnList.AddChild(tnUnit, tmpClass.FName);
      tnClass.Data := tmpClass;
    end;
  end;
end;


{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCStructure object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, there is only one reference per application.

  @param  st  Name of the new Strcutre (mainly its identifier)
  @param  rp  Related logical parent and owner, generally a null value !
-------------------------------------------------------------------------------}
constructor TDOCStructure.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  FName := 'Project1';
  Units := TDOCUnitList.Create(True);
end;

//  Destroys of an instance of a TDOCStructure object and free its content
destructor TDOCStructure.Destroy;
begin
  FreeAndNil(Units);
  inherited;
end;

{-------------------------------------------------------------------------------
  Find a Unit reference in the structure by its name
  It simply call the FindItemByName of the list.
  @param  uname Name of the Unit to find
  @return Unit reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCStructure.FindUnit(const uname: string): TDOCTemplate;
begin
  Result := Units.FindItemByName(uname);
end;

{-------------------------------------------------------------------------------
  Get the reference of the Unit specified by its index in the Units List
  @param  index Index in the Units List
  @return Unit reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCStructure.GetUnit(const index: integer): TDOCTemplate;
begin
  Result := Units.GetItem(index);
end;

// Sort all DocTemplate element of lists in alphabetical order
procedure TDOCStructure.SortContent;
var
  Index : integer;
begin
  Units.Sort(DocTemplateListSortCompare);
  for Index:=0 to Units.Count-1 do
   TDOCUnit(Units.Items[Index]).SortContent;
end;

{-------------------------------------------------------------------------------
  Return number of Units in the list
  @return Number of methods
-------------------------------------------------------------------------------}
function TDOCStructure.UnitsCount: integer;
begin
  Result := Units.Count;
end;

{ TDOCTemplateList }

{-------------------------------------------------------------------------------
  Find a TDOCTemplate reference in the list by its name
  @param  st Name of the item to find
  @return Item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.FindItemByName(const st: string): TDOCTemplate;
var
  Tmp: TDOCTemplate;
  Index: integer;
begin
  Result := nil;
  Index  := 1;
  while ((Index <= Count) and (Result = nil)) do
  begin
    Tmp := Items[Index -1];
    if Assigned(Tmp) then
      if Tmp.FName = st then
        Result := Tmp;
    inc(Index);
  end;
end;

{-------------------------------------------------------------------------------
  Get the reference of the DocTemplate specified by its index in the Items List
  @param  index Index in the DocTemplate List
  @return DocTemplate reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.GetItem(const Index: integer): TDOCTemplate;
begin
  Result := inherited GetItem(Index) as TDOCTemplate;
end;

{-------------------------------------------------------------------------------
  Get the next DocTemplate item in the DocTemplate List
  @param  Item  Base item to find the next one
  @return DocTemplate Next item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.NextItem(const Item: TDOCTemplate): TDOCTemplate;
var
  Index: Integer;
begin
  Index:= IndexOf(Item);
  if Index<(count-1) then Result:=Items[Index+1]
    else Result:=nil;
end;

{-------------------------------------------------------------------------------
  Get the previous DocTemplate item in the DocTemplate List
  @param  Item  Base item to find the previous one
  @return DocTemplate Previous item reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCTemplateList.PrevItem(const Item: TDOCTemplate): TDOCTemplate;
var
  Index: Integer;
begin
  Index:= IndexOf(Item);
  if Index>0 then Result:=Items[Index-1]
    else Result:=nil;
end;

procedure TDOCTemplateList.SetItem(const Index: integer; const item: TDOCTemplate);
begin
  inherited SetItem(index, item as TObject);
end;

{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCLink object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  ln  Name of the new Link (mainly its identifier)
-------------------------------------------------------------------------------}
constructor TDOCLink.Create(const ln: string);
begin
  Name := ln;
  Link := nil;
end;

{-------------------------------------------------------------------------------
  Add a Parameter to the structure
  @param  cname New Parameter name (its identifier)
  @return Newly created reference for the class
-------------------------------------------------------------------------------}
function TDOCMethod.AddParameter(const pname: string): TDOCTemplate;
begin
  Result := TDOCParameter.Create(pname, self);
  Parameters.Add(Result);
end;

{-------------------------------------------------------------------------------
  Call Create to instantiate a TDOCMethod object at runtime.
  Create allocates memory, and then initializes its properties.
  Generally, The owner of this item call create and add the reference in a list

  @param  st  Name of the new Method (mainly its identifier)
  @param  rp  Related logical parent and owner, generally the class it depend
-------------------------------------------------------------------------------}
constructor TDOCMethod.Create(const st: string; const rp: TDocTemplate);
begin
  inherited;
  Parameters := TDOCParameterList.Create(True);
end;

//  Destroys of an instance of a TDOCMethod object
destructor TDOCMethod.Destroy;
begin
  FreeAndNil(Parameters);
  inherited;
end;

{-------------------------------------------------------------------------------
  Get the reference of the Parameter specified by its index in the Parameters List
  @param  index Index in the Parameters List
  @return Parameter reference if successfull, nil otherwise
-------------------------------------------------------------------------------}
function TDOCMethod.GetParameter(const index: integer): TDOCTemplate;
begin
  Result := Parameters.GetItem(index);
end;

{-------------------------------------------------------------------------------
  Return number of Parameters in the list
  @return Number of Parameters
-------------------------------------------------------------------------------}
function TDOCMethod.ParametersCount: integer;
begin
  Result := Parameters.Count;
end;

initialization
  MemberVisibilityName := TStringList.Create;
  MemberVisibilityName.Add('PRIVATE');
  MemberVisibilityName.Add('PROTECTED');
  MemberVisibilityName.Add('PUBLIC');
  MemberVisibilityName.Add('PUBLISHED');
  MemberVisibilityName.Add('AUTOMATED');

finalization
  FreeAndNil(MemberVisibilityName);
end.
