{-------------------------------------------------------------------------------
  Module that centralize all Options.
  This module contains the TDOCOptions, and all informations for each option :
   Name, Description, Values, translations, etc ...
  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit uOptions;

interface

uses
  Classes, SysUtils;

type
  TOutputFormat  = (ofTEXT, ofHTML, ofCHM, ofPDF);
  TMemberVisibility = (mvPrivate, mvProtected, mvPublic, mvPublished, mvAutomated);
  TVisibilitySet = set of TMemberVisibility;
  TTagComment    = (tcINLINE, tcJAVADOC, tcJEDI);
  TTagCommentSet = set of TTagComment;
  TLanguage      = (lEnglish, lFrench);

  { Holder for all Options module
    Parser, structure builder, Generator and the DelphiCodeToDoc application
    share this class that contains all options.}
  TDOCOptions = class(TCOmponent) // Must be a TComponent to have published property
  private
    //Property Field
    FRecurseDirectory: boolean; // Search inside sub-directoryes
    FTitle: string; // Title of the project
    FHTMLlink: string;  // HTML link to the project page, or author page
    FCopyright: string; // Copyright of the author
    FAuthor: string;  // Information about the author (name, email for example)
    FName: string;  // Name of the project, use as a basename for the output files
    FCustomFooter: TFileName; // Author's Footer filename insert in each generated files
    FOutputFolder: TFileName; // Where all output files will be saved
    FCustomHeader: TFileName; // Author's Header filename insert in each generated files
    FRootDirectory: TFileName;  // The directory where all sources files will be taken from
    FLanguage: TLanguage; // Ouput Language of the documentation
    FOutputFormat: TOutputFormat; // Output format od the documentation
    FConditionalDefines: TStringList; // List of defines needed to interprete source code
    FSummary: TStringList;  // Summary of the project (few lines are ok)
    FTagCommentSet: TTagCommentSet; //  TAG that will be extracted to the documentation
    FVisibilitySet: TVisibilitySet; // Visibility filter wanted for the output
    FBoolsAsChecks: boolean;  //  See Checkmarks or boolean expression for options
    FLaunchBrowser: boolean;  // Launch browser at the end of the documentation generation
    FAlphaSort: boolean;  // Alphabetical sort of the Unit, Class, and member name
    FWarnEmptyTAG: Boolean; // Add a warning message in the documentation if some TAGs are missing
    FHideEmptyTAGSection: boolean;  // Empty TAG section will be hide
    FSkipFirstTAGLine: boolean; // Skip first TAG line, that are sometimes decorator
    FSkipLastTAGLine: boolean;  // Skip last TAG line, that are sometimes decorator
    FParseImplementationSection: boolean; // Also Parse implementation section (for TAG)
  published
    // General options
    // Application Options
    property BoolsAsChecks: boolean read FBoolsAsChecks write FBoolsAsChecks;
    property LaunchBrowser: boolean read FLaunchBrowser write FLaunchBrowser;
    // Input options
    property Name: string read FName write FName;
    property Title: string read FTitle write FTitle;
    property Summary: TStringList read FSummary write FSummary;
    property Author: string read FAuthor write FAuthor;
    property HTMLlink: string read FHTMLlink write FHTMLlink;
    property Copyright: string read FCopyright write FCopyright;
    property RootDirectory: TFileName read FRootDirectory write FRootDirectory;
    property RecurseDirectory: boolean read FRecurseDirectory write FRecurseDirectory;
    property ConditionalDefines: TStringList read FConditionalDefines write FConditionalDefines;
    property ParseImplementationSection: boolean read FParseImplementationSection write FParseImplementationSection;
    // Output options
    property CustomHeader: TFileName read FCustomHeader write FCustomHeader;
    property CustomFooter: TFileName read FCustomFooter write FCustomFooter;
    property Language: TLanguage read FLanguage write FLanguage;
    property OutputFormat: TOutputFormat read FOutputFormat write FOutputFormat;
    property OutputFolder: TFileName read FOutputFolder write FOutputFolder;
    property VisibilitySet: TVisibilitySet read FVisibilitySet write FVisibilitySet;
    property TagCommentSet: TTagCommentSet read FTagCommentSet write FTagCommentSet;
    property AlphaSort : boolean read FAlphaSort write FAlphaSort;
    property WarnEmptyTAG: Boolean read FWarnEmptyTAG write FWarnEmptyTAG;
    property HideEmptyTAGSection: boolean read FHideEmptyTAGSection write FHideEmptyTAGSection;
    // JavaDoc Support
    property SkipFirstTAGLine: boolean read FSkipFirstTAGLine write FSkipFirstTAGLine;
    property SkipLastTAGLine: boolean read FSkipLastTAGLine write FSkipLastTAGLine;
  public
    procedure SetDefaultValue;
    constructor Create(AOwner: TComponent);override;
    destructor destroy;override;
  end;

  { Each option need to be describe as follow :
    The name of the property inside the class : pdName_____
    The written Caption of this options in the application : pdCaption_____
    The possibility for the user to modify this option at runtime : TRUE/FALSE
    A notify method to react immediately about a changed value for this option}
    TPropertyDescription = record
    iName, iCaption: string;
    iReadOnly: boolean;
    iChanged: TNotifyEvent;
  end;

resourcestring
  // Caption and name for Options
  pdNameBoolsAsChecks = 'BoolsAsChecks';
  pdCaptionBoolsAsChecks = 'Bools as Checks';
  pdNameName = 'Name';
  pdCaptionName = 'Project''s Name';
  pdNameTitle = 'Title';
  pdCaptionTitle = 'Project''s Title';
  pdNameSummary = 'Summary';
  pdCaptionSummary = 'Project''s Summary';
  pdNameAuthor = 'Author';
  pdCaptionAuthor = 'Author';
  pdNameHTMLLink = 'HTMLlink';
  pdCaptionHTMLLink = 'HTMLlink';
  pdNameCopyright = 'Copyright';
  pdCaptionCopyright = 'Author''s Copyright';
  pdNameRootDirectory = 'RootDirectory';
  pdCaptionRootDirectory = 'RootDirectory';
  pdNameRecurseDirectory = 'RecurseDirectory';
  pdCaptionRecurseDirectory = 'RecurseDirectory';
  pdNameConditionalDefines = 'ConditionalDefines';
  pdCaptionConditionalDefines = 'ConditionalDefines';
  pdNameCustomHeader = 'CustomHeader';
  pdCaptionCustomHeader = 'CustomHeader';
  pdNameCustomFooter = 'CustomFooter';
  pdCaptionCustomFooter = 'CustomFooter';
  pdNameLanguage = 'Language';
  pdCaptionLanguage = 'Language';
  pdNameOutputFormat = 'OutputFormat';
  pdCaptionOutputFormat = 'OutputFormat';
  pdNameOutputFolder = 'OutputFolder';
  pdCaptionOutputFolder = 'OutputFolder';
  pdNameVisibilitySet = 'VisibilitySet';
  pdCaptionVisibilitySet = 'VisibilitySet';
  pdNameTagCommentSet = 'TagCommentSet';
  pdCaptionTagCommentSet = 'TagCommentSet';
  pdNameLaunchBrowser = 'LaunchBrowser';
  pdCaptionLaunchBrowser = 'Launch Browser after Build?';
  pdNameSortAlpha = 'AlphaSort';
  psCaptionSortAlpha = 'Alpha sort of members';
  pdNameWarnEmptyTAG = 'WarnEmptyTAG';
  psCaptionWarnEmptyTAG = 'Warn when TAG are empty';
  pdNameHideEmptyTAGSection = 'HideEmptyTAGSection';
  pdCaptionHideEmptyTAGSection = 'Hide empty TAG section from report';
  pdNameSkipFirstTAGLine = 'SkipFirstTAGLine';
  pdCaptionSkipFirstTAGLine = 'Skip first TAG line in comment block';
  pdNameSkipLastTAGLine = 'SkipLastTAGLine';
  pdCaptionSkipLastTAGLine = 'Skip last TAG line in comment block';
  pdNameParseImplementationSection = 'ParseImplementationSection';
  pdCaptionParseImplementationSection = 'Parse Implementation Section';
  // Catpion for Custom Category
  ccApplicationOptions = 'Application options';
  ccInputOptions = 'Input options';
  ccOutputOptions = 'Output options';
  ccJavaDocSupportOptions = 'JavaDoc support options';


const
  C_APPLICATION_PROPERTYS_DESCRIPTION: array[1..2] of TPropertyDescription =
    ( (iName: pdNameBoolsAsChecks; iCaption: pdCaptionBoolsAsChecks; iReadOnly: False)
    , (iName: pdNameLaunchBrowser; iCaption: pdCaptionLaunchBrowser; iReadOnly: False)
    );

  C_INPUT_PROPERTYS_DESCRIPTION: array[1..4] of TPropertyDescription =
    ( (iName: pdNameName; iCaption: pdCaptionName; iReadOnly: False)
    //, (iName: pdNameTitle; iCaption: pdCaptionTitle; iReadOnly: False)
    //, (iName: pdNameSummary; iCaption: pdCaptionSummary; iReadOnly: False)
    //, (iName: pdNameAuthor; iCaption: pdCaptionAuthor; iReadOnly: False)
    //, (iName: pdNameHTMLLink; iCaption: pdCaptionHTMLLink; iReadOnly: False)
    //, (iName: pdNameCopyright; iCaption: pdCaptionCopyright; iReadOnly: False)
    , (iName: pdNameRootDirectory; iCaption: pdCaptionRootDirectory; iReadOnly: True)
    , (iName: pdNameRecurseDirectory; iCaption: pdCaptionRecurseDirectory; iReadOnly: False)
    //, (iName: pdNameConditionalDefines; iCaption: pdCaptionConditionalDefines;iReadOnly: False)
    , (iName: pdNameParseImplementationSection; iCaption: pdCaptionParseImplementationSection;iReadOnly: true)
    );

  C_OUTPUT_PROPERTYS_DESCRIPTION: array[1..8] of TPropertyDescription =
    ( //(iName: pdNameCustomHeader; iCaption: pdCaptionCustomHeader; iReadOnly: True)
    //, (iName: pdNameCustomFooter; iCaption: pdCaptionCustomFooter; iReadOnly: True)
     (iName: pdNameLanguage; iCaption: pdCaptionLanguage; iReadOnly: True)
    , (iName: pdNameOutputFormat; iCaption: pdCaptionOutputFormat; iReadOnly: True)
    , (iName: pdNameOutputFolder; iCaption: pdCaptionOutputFolder; iReadOnly: True)
    , (iName: pdNameVisibilitySet; iCaption: pdCaptionVisibilitySet; iReadOnly: False)
    , (iName: pdNameTagCommentSet; iCaption: pdCaptionTagCommentSet; iReadOnly: True)
    , (iName: pdNameSortAlpha; iCaption: psCaptionSortAlpha; iReadOnly: False)
    , (iName: pdNameWarnEmptyTAG ; iCaption: psCaptionWarnEmptyTAG ; iReadOnly: False)
    , (iName: pdNameHideEmptyTAGSection ; iCaption: pdCaptionHideEmptyTAGSection ; iReadOnly: False)
    );

  C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION: array[1..2] of TPropertyDescription =
    ((iName: pdNameSkipFirstTAGLine; iCaption: pdCaptionSkipFirstTAGLine; iReadOnly: False)
    ,(iName: pdNameSkipLastTAGLine; iCaption: pdCaptionSkipLastTAGLine; iReadOnly: False)
    );

implementation

// Creates a TDOCOptions reference
constructor TDOCOptions.Create(AOwner: TComponent);
begin
  FSummary      := TStringList.Create;
  FConditionalDefines := TStringList.Create;
end;

// Destroy a TDOCOption reference
destructor TDOCOptions.destroy;
begin
  FreeAndNil(FConditionalDefines);
  FreeAndNil(FSummary);
end;

// Set all options to a default value
procedure TDOCOptions.SetDefaultValue;
begin
  FTitle         := 'Insert your project''s title here';
  FHTMLlink      := 'http://dephicodetodoc.sf.net (Replace by your webpage for example)';
  FCopyright     := 'TridenT - 2003 - Under GNU GPL Licence';
  FAuthor        := 'TridenT';
  FName          := 'Project1';
  FCustomFooter  := 'UNDER CONSTRUCTION';
  FOutputFolder  := '.\out\';
  FCustomHeader  := 'UNDER CONSTRUCTION';
  FRootDirectory := 'Enter your project''s root directory';
  FLanguage      := lEnglish;
  FOutputFormat  := ofHTML;
  //FConditionalDefines: TStringList;
  //FSummary: TStringList;
  FAlphaSort:= true;
  FTagCommentSet := [tcInline,tcJavaDoc];
  FVisibilitySet := [mvPrivate, mvProtected, mvPublic, mvPublished, mvAutomated];
  FBoolsAsChecks := True;
  FRecurseDirectory := True;
  FLaunchBrowser := True;
  FWarnEmptyTAG := True;
  FHideEmptyTAGSection := False;
  FSkipFirstTAGLine:= true;
  FSkipLastTAGLine:= true;
  FParseImplementationSection:= true;
end;



end.
