{-------------------------------------------------------------------------------
  User interface of DelphiCodeToDoc

  http://dephicodetodoc.sourceforge.net/
  Copyright TridenT 2003 - Under GNU GPL licence

  @Author    TridenT
  @Version   2003/12/01   TridenT   v0.0     Initial revision
-------------------------------------------------------------------------------}

unit fMainForm;

interface

uses
  Classes, forms, ImgList, Controls, StdActns, JvDialogActns, ActnList,
  XPStyleActnCtrls, ActnMan, ActnMenus, ToolWin, ActnCtrls, ComCtrls,
  JvStatusBar, JvComponent, JvLookOut, SysUtils, StdCtrls, JvInspector,
  JvComCtrls;

type
  TMainForm = class(TForm)
    JvExpress1: TJvExpress;
    ebInformation: TJvExpressButton;
    ebConfiguration: TJvExpressButton;
    ebProjectFiles: TJvExpressButton;
    ActionToolBar1: TActionToolBar;
    ebReference: TJvExpressButton;
    ebMessages: TJvExpressButton;
    ImageListSmall: TImageList;
    ImageListLarge: TImageList;
    ActionManager1: TActionManager;
    ActionNewproject: TAction;
    ActionWizardProject: TAction;
    JvOpenProject: TJvOpenFileAction;
    JvActionSaveProjectAs: TJvSaveFileAction;
    ActionSaveAll: TAction;
    FileExit1: TFileExit;
    ActionAbout: TAction;
    ActionAddFiles: TAction;
    ActionCheck: TAction;
    ActionBuild: TAction;
    ActionInformation: TAction;
    ActionConfiguration: TAction;
    ActionProject: TAction;
    ActionReference: TAction;
    ActionMessages: TAction;
    ActionMainMenuBar1: TActionMainMenuBar;
    StatusBar1: TStatusBar;
    JvPageControl1: TJvPageControl;
    tsInformation: TTabSheet;
    tsConfiguration: TTabSheet;
    tsProjectFiles: TTabSheet;
    tsReference: TTabSheet;
    tsMessages: TTabSheet;
    Memo1: TMemo;
    mDebug: TMemo;
    JvInspector1: TJvInspector;
    JvInspectorDotNETPainter1: TJvInspectorDotNETPainter;
    mUser: TMemo;
    tvReference: TJvTreeView;
    LabelMustCheckBefore: TLabel;
    ActionCloseProject: TAction;
    procedure ActionAboutExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ActionInformationExecute(Sender: TObject);
    procedure ActionConfigurationExecute(Sender: TObject);
    procedure ActionProjectExecute(Sender: TObject);
    procedure ActionReferenceExecute(Sender: TObject);
    procedure ActionMessagesExecute(Sender: TObject);
    procedure ActionWizardProjectExecute(Sender: TObject);
    procedure ActionCheckExecute(Sender: TObject);
    procedure ActionBuildExecute(Sender: TObject);
    procedure ActionCloseProjectExecute(Sender: TObject);
    procedure ActionManager1Update(Action: TBasicAction;
      var Handled: Boolean);
  private
    { Private declarations }
    procedure AddConfigurationPropertys;
    procedure ChangeChkState(const Item: TJvCustomInspectorItem);
    procedure iieBoolsAsChecks(Sender: TObject);
    procedure UpdateReferences;
  public
    { Public declarations }
    procedure AddUserMessage(const st: string);
    procedure AddDebugMessage(const st: string);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses fAbout, uDocProject, JvInspExtraEditors, TypInfo, fWizardProject
    , JvMemo, JclShell, uOptions;

resourcestring
  // User Messages
  L_WizardSuccess = 'Wizard successfully create the project';
  L_WizardCanceled = 'Wizard operation Canceled';
  L_CheckedSuccess = 'Successfully Checked the project, BUILD function is now enable';
  L_CheckingAborted = 'Project checking aborted';
  L_BuildSuccess = 'Successfully build the Project';
  L_LaunchBrowser = 'Launching browser to display result';
  L_ConfigurationChanged = 'Project configuration changed. You need to re-CHECK project';
  L_ProjectClosed = 'Project closed. To start a new project, try the NEW WIZARD PROJECT';

procedure TMainForm.ActionAboutExecute(Sender: TObject);
var 
  AboutBox: TAboutBox;
begin
  AboutBox := TAboutBox.Create(self);
  AboutBox.ShowModal;
  AboutBox.Release;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  self.Caption := Application.Title;
  ActionInformationExecute(self);
  // Set divider to 50% of inspector's width
  JvInspector1.Divider := JvInspector1.Width div 2;
end;

procedure TMainForm.ActionInformationExecute(Sender: TObject);
begin
  ebInformation.Down := True;
  tsInformation.Show;
end;

procedure TMainForm.ActionConfigurationExecute(Sender: TObject);
begin
  ebConfiguration.Down := True;
  tsConfiguration.Show;
end;

procedure TMainForm.ActionProjectExecute(Sender: TObject);
begin
  ebProjectFiles.Down := True;
  tsProjectFiles.Show;
end;

procedure TMainForm.ActionReferenceExecute(Sender: TObject);
begin
  ebReference.Down := True;
  tsReference.Show;
end;

procedure TMainForm.ActionMessagesExecute(Sender: TObject);
begin
  ebMessages.Down := True;
  tsMessages.Show;
end;

procedure TMainForm.ChangeChkState(const Item: TJvCustomInspectorItem);
var
  I: integer;
begin
  if Item is TJvInspectorBooleanItem then
    TJvInspectorBooleanItem(Item).ShowAsCheckbox := DocProject.DocOptions.BoolsAsChecks;
  for I := 0 to Item.Count - 1 do
    ChangeChkState(Item[I]);
end;

function AddProperty(Cat: TJvInspectorCustomCategoryItem;
  Prop: TPropertyDescription): TJvCustomInspectorItem;
begin
  Result:= TJvInspectorPropData.New(Cat, DocProject.DocOptions
   , GetPropInfo(DocProject.DocOptions, Prop.iName));
  Result.ReadOnly := Prop.iReadOnly;
  if Prop.iReadOnly then Result.DisplayName := Prop.iCaption+' (ReadOnly)'  //'*'
  else Result.DisplayName := Prop.iCaption;

  if Prop.iName = pdNameBoolsAsChecks then Result.OnValueChanged :=
      MainForm.iieBoolsAsChecks;
  MainForm.ChangeChkState(Result);
end;

procedure TMainForm.AddConfigurationPropertys;
var
  InspCat: TJvInspectorCustomCategoryItem;
  IndexProp: integer;
begin
  JvInspector1.BeginUpdate;
  JvInspector1.Root.SortKind := iskNone;
  JvInspector1.SelectedIndex := 0;
  // APPLICATION
  InspCat          := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCat.DisplayName := ccApplicationOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_APPLICATION_PROPERTYS_DESCRIPTION)
    to high(C_APPLICATION_PROPERTYS_DESCRIPTION) do
    AddProperty(InspCat, C_APPLICATION_PROPERTYS_DESCRIPTION[IndexProp]);
  // INPUT
  InspCat          := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCat.DisplayName := ccInputOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_INPUT_PROPERTYS_DESCRIPTION)
    to high(C_INPUT_PROPERTYS_DESCRIPTION) do
    AddProperty(InspCat, C_INPUT_PROPERTYS_DESCRIPTION[IndexProp]);
  // OUTPUT
  InspCat          := TJvInspectorCustomCategoryItem.Create(JvInspector1.Root, nil);
  InspCat.DisplayName := ccOutputOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_OUTPUT_PROPERTYS_DESCRIPTION)
    to high(C_OUTPUT_PROPERTYS_DESCRIPTION) do
    AddProperty(InspCat, C_OUTPUT_PROPERTYS_DESCRIPTION[IndexProp]);
  // JAVADOC SUPPORT
  InspCat          := TJvInspectorCustomCategoryItem.Create(InspCat, nil);
  InspCat.DisplayName := ccJavaDocSupportOptions;
  InspCat.Expanded := True;
  for IndexProp := low(C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION)
    to high(C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION) do
    AddProperty(InspCat, C_JAVADOC_SUPPORT_PROPERTYS_DESCRIPTION[IndexProp]);
  // End of build
  JvInspector1.EndUpdate;
end;

procedure TMainForm.iieBoolsAsChecks(Sender: TObject);
begin
  JvInspector1.BeginUpdate;
  try
    ChangeChkState(JvInspector1.Root);
  finally
    JvInspector1.EndUpdate;
  end;
end;

procedure TMainForm.ActionWizardProjectExecute(Sender: TObject);
begin
  WizardProjectForm := TWizardProjectForm.Create(self);
  if WizardProjectForm.ShowModal = mrOk then
  begin
    DocProject := TDOCProject.Create;
    DocProject.DocOptions.SetDefaultValue;
    DocProject.DocOptions.RootDirectory := WizardProjectForm.GetRootDirectory;
    DocProject.DocOptions.OutputFolder  := WizardProjectForm.GetOutputDirectory;
    DocProject.DocOptions.Name          := WizardProjectForm.GetProjectName;
    AddConfigurationPropertys;
    DocProject.SetDestinationMessages(mUser.Lines, mDebug.Lines);
    //ActionCheck.Enabled := True;
    AddUserMessage(L_WizardSuccess);
    //ActionWizardProject.Enabled := False;
    DocProject.State:=psNew;
    // Go to the Messages panel
    ActionConfigurationExecute(self);
  end
  else 
    AddDebugMessage(L_WizardCanceled);
  WizardProjectForm.Release;
end;

procedure TMainForm.ActionCheckExecute(Sender: TObject);
begin
  // Check if Project can be built
  if DocProject.Check then
  begin
    LabelMustCheckBefore.Visible := False;
    // UpdateReference
    UpdateReferences;
    tvReference.Visible := True;
    //ActionBuild.Enabled := True;
    AddUserMessage(L_CheckedSuccess);
    // Go to the Messages panel
    ActionMessages.Execute;
  end
  else
    AddUserMessage(L_CheckingAborted);
end;

procedure TMainForm.ActionBuildExecute(Sender: TObject);
begin
  // Build Project documentation
  Assert(DocProject.isChecked);
  DocProject.Build;
  // Verify if all files are built !
  AdduserMessage(L_BuildSuccess);
  // Go to the Messages panel
  ActionMessagesExecute(self);
  // Launch the browser ?
  if DocProject.DocOptions.LaunchBrowser then
  begin
    AddUserMessage(L_LaunchBrowser);
    ShellExecEx(DocProject.DocOptions.OutputFolder + DocProject.DocOptions.Name + '.htm');
  end;
end;

procedure TMainForm.AddDebugMessage(const st: string);
begin
  mDebug.Lines.Add(st);
end;

procedure TMainForm.AddUserMessage(const st: string);
begin
  mUser.Lines.Add(st);
  StatusBar1.SimpleText:=st;
end;

procedure RecurseSetCheck(const tn: TTreeNode; const Value: boolean);
var
  Index: integer;
  jvtn: TJvTreeNode;
begin
  assert(tn is TJvTreeNode);
  jvtn         := tn as TJvTreeNode;
  jvtn.Checked := Value;
  // Issue : Checkmarks doesn't work !!!
  //jvtn.Bold:=Value;
  for Index := 0 to jvtn.Count - 1 do
    RecurseSetCheck(jvtn.Item[Index], Value);
end;

// List project's units and classes in a TreeView
procedure TMainForm.UpdateReferences;
begin
  // TODO: save reference informations if exists
  // Clear top Reference if exist  (inside convert function)
  //if tvReference.TopItem <> nil then tvReference.TopItem.Delete;
  DocProject.GetStructure.ConvertToTreeNode(tvReference.Items);
  tvReference.FullExpand;
  // Add checkmark to all Nodes
  RecurseSetCheck(tvReference.Items.GetFirstNode, True);
  // TODO : retrieve reference information if exist
end;

procedure TMainForm.ActionCloseProjectExecute(Sender: TObject);
begin
  // Free Configuration list
  JvInspector1.Root.Clear;
  // Close project
  if Assigned(DocProject) then FreeAndNil(DocProject);
  tvReference.Items.Clear;
  AddUserMessage(L_ProjectClosed);
end;

procedure TMainForm.ActionManager1Update(Action: TBasicAction;
  var Handled: Boolean);
begin
  if assigned(DocProject) then
  case DocProject.State of
    psNew:
    begin
      ActionWizardProject.Enabled:=false;
      ActionCloseProject.Enabled:=true;
      ActionCheck.Enabled:=true;
      ActionBuild.Enabled:=false;
    end;
    psChecked:
    begin
      ActionWizardProject.Enabled:=false;
      ActionCloseProject.Enabled:=true;
      ActionCheck.Enabled:=true;
      ActionBuild.Enabled:=true;
    end;
    psBuilt:
    begin
      ActionWizardProject.Enabled:=false;
      ActionCloseProject.Enabled:=true;
      ActionCheck.Enabled:=true;
      ActionBuild.Enabled:=true;
    end;
  else // Unknown state !
    begin
      ActionWizardProject.Enabled:=false;
      ActionCloseProject.Enabled:=false;
      ActionCheck.Enabled:=false;
      ActionBuild.Enabled:=false;
    end;
  end
  else // Project doesn't exist
  begin
    ActionWizardProject.Enabled:=true;
    ActionCloseProject.Enabled:=false;
    ActionCheck.Enabled:=false;
    ActionBuild.Enabled:=false;
  end;
  handled:=true;
end;

initialization
  TJvInspectorColorItem.RegisterAsDefaultItem;
  TJvInspectorTImageIndexItem.RegisterAsDefaultItem;
end.
